#ifndef _DATA_H_
#define _DATA_H_

#include <stddef.h>
#include <stdint.h>

#define DATA_INDEX_PAYLOAD	       2
#define GSENSOR_ALL_PACKETS_LENGTH 182
#define GSENSOR_HEADER_ST1         0xFE
#define GSENSOR_HEADER_ST2         0xFD
#define GSENSOR_ONE_PACKET_LENGTH  30
#define PPG_ALL_PACKETS_LENGTH     182
#define PPG_HEADER_ST1             0xEE
#define PPG_HEADER_ST2             0xED
#define PPG_ONE_PACKET_LENGTH      20
#define LENGTH_BUF_LEN             6

typedef enum {
    PPG_CHANNEL_G = 0,
    PPG_CHANNEL_R,    
    PPG_CHANNEL_IR
} ppg_channel_t;

int      dataCompare(const size_t length, const uint8_t* pData1, const uint8_t* pData2);
int      dataFindHeaderCount(const size_t length, const uint8_t* pData);
int      dataFindGsensorHeaderCount(const size_t length, const uint8_t* pData);
int      dataFindPpgHeaderCount(const size_t length, const uint8_t* pData);
int      dataFindGsensorHeaderIndex(const size_t length, const uint8_t* pData, uint8_t* pIdx);
int      dataFindPpgHeaderIndex(const size_t length, const uint8_t* pData, uint8_t* pIdx);
int      dataFindHeaderIndex(const size_t length, const uint8_t* pData);
int      dataFindMaxEraseByteCount(const size_t length, const uint8_t* pData);
void     data2Length(const uint8_t* pData, uint32_t* pLen);
void     length2Data(const uint32_t length, uint8_t* pData);
uint32_t gsensorData2Uint32(const uint8_t* pData);
uint32_t ppgData2Uint32(const uint8_t* pData);
int      getPacketsNum(const int len);
void     dataPrintHex(const char* str, const uint16_t len, const uint8_t* pData);
void     dataPrintChar(const char* str, const uint16_t len, const uint8_t* pData);
void     ppgDataPrint(const char* str,
                       const ppg_channel_t ch,
                       const int idx,
                       const uint16_t len,
                       const uint8_t* pData);


#endif /* _DATA_H_ */

