/*
 * AFE4900.c
 *
 * Provides AFE4900 API
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED  
 *
*/

#include <inttypes.h>
#include <nrfx_gpiote.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <zephyr/sys/util.h>
#include <zephyr/sys/printk.h>
#include "button.h"
#include "error.h"
#include "led.h"
#include "localtime.h"
#include "nrf52832_board.h"
#include "pwr_mgmt.h"

#define BUTTON_TIMER_MSEC 10000

/*
 * Get configuration from the devicetree alias. This is mandatory.
 */
#define AFE4900_RDY_NODE DT_ALIAS(afe4900rdy)

#if !DT_NODE_HAS_STATUS(AFE4900_RDY_NODE, okay)
#error "Unsupported board: afe4900rdy devicetree alias is not defined"
#endif

LOG_MODULE_REGISTER(afe4900);

/*
 * Helper macro for initializing a gpio_dt_spec from the devicetree
 * with fallback values when the nodes are missing.
 */
static const struct gpio_dt_spec _gAfe4900Rdy   = GPIO_DT_SPEC_GET_OR(AFE4900_RDY_NODE, gpios, {0});

//static struct gpio_callback button_cb_data;
static void _afe4900RdyHandler(nrfx_gpiote_pin_t     pin,
                          nrfx_gpiote_trigger_t trigger,
                          void *                p_context) {
    printk(" _afe4900RdyHandler() enter\n");

}

int afe4900Init(void) {
    LOG_INF("button long press test\n");
    printk("Starting button Test example\n");
    //int ret;
    nrfx_err_t errCode;

    ledInit();

    if (!device_is_ready( _gAfe4900Rdy.port)) {
        printk("Error: button device %s is not ready\n", _gAfe4900Rdy.port->name);
        return ERROR_BUTTON_INITIALIZE_FAIL;
    }

    printk("device_is_ready() ok\n");
#if 0
    nrfx_gpiote_in_config_t inConfig = NRFX_GPIOTE_CONFIG_IN_SENSE_TOGGLE(true);
    inConfig.sense = NRF_GPIOTE_POLARITY_LOTOHI;
    inConfig.pull  = NRF_GPIO_PIN_PULLDOWN; // interrpt: raising edge
    errCode = nrfx_gpiote_in_init(BUTTON_1, &inConfig, button_pressed);
    
    int val = gpio_pin_get_dt(&_gButton);
    printk("button state=%d\n", val);
    //if (val == BUTTON_PRESS) 
    {
        printk("start a timer\n");
        ltPrintCurrentDatetime();
        printk("%" PRId64 "\n", k_uptime_get());
        k_timer_start(&_gButtonTimer, K_MSEC(BUTTON_TIMER_MSEC), K_NO_WAIT);
    }
#else
	nrfx_gpiote_input_config_t inputConfig = {.pull = NRF_GPIO_PIN_PULLUP};
	
	uint8_t gpioteChannel;
	errCode = nrfx_gpiote_channel_alloc(&gpioteChannel);
	nrfx_gpiote_trigger_config_t triggerConfig = {
		.trigger 		= NRFX_GPIOTE_TRIGGER_TOGGLE, 
		.p_in_channel 	= &gpioteChannel
	};

	nrfx_gpiote_handler_config_t handleConfig = {
		.handler 	= &_afe4900RdyHandler,
		.p_context 	= NULL
	};
	
	errCode = nrfx_gpiote_input_configure(_gAfe4900Rdy.pin, &inputConfig, &triggerConfig, &handleConfig);
	if (errCode != NRFX_SUCCESS) {
		printk("Error: nrfx_gpiote_input_configure() failed for AFE4900_RDY, errCode=0x%x\n", errCode);
        return ERROR_BUTTON_INITIALIZE_FAIL;
	} else {
		printk("nrfx_gpiote_input_configure() okay for AFE4900_RDY\n");
    }
	
	nrfx_gpiote_trigger_enable(_gAfe4900Rdy.pin, true);

   
#endif	
    return 0;
}
