#include <zephyr/sys/printk.h>
#include "data.h"

// length
#define LENGTH_HEADER_ST1        0xCE
#define LENGTH_HEADER_ST2        0xCD
#define LENGTH_IDX_DATA_START    2
#define TIME_HEADER_ST1          0xBE
#define TIME_HEADER_ST2          0xBD

int dataCompare(const size_t length, const uint8_t* pData1, const uint8_t* pData2) {
    for (int i=0; i<length; i++) {
        if (pData1[i] != pData2[i]) {
            return -1;
        }    
    }

    return 0;
}

int dataFindHeaderCount(const size_t length, const uint8_t* pData) {
    if (length == 0) {
        return 0;
    }

    int count = 0;
    for (int i=0; i<length-1; i++) {
        if (((pData[i] == GSENSOR_HEADER_ST1) && (pData[i+1] == GSENSOR_HEADER_ST2)) ||
            ((pData[i] == PPG_HEADER_ST1)     && (pData[i+1] == PPG_HEADER_ST2))) {
            count++;
        }    
    }

    return count;
}

int dataFindGsensorHeaderCount(const size_t length, const uint8_t* pData) {
    if (length == 0) {
        return 0;
    }

    int count = 0;
    for (int i=0; i<length-1; i++) {
        if ((pData[i] == GSENSOR_HEADER_ST1) && (pData[i+1] == GSENSOR_HEADER_ST2)) {
            count++;
        }    
    }

    return count;
}

int dataFindPpgHeaderCount(const size_t length, const uint8_t* pData) {
    if (length == 0) {
        return 0;
    }

    int count = 0;
    for (int i=0; i<length-1; i++) {
        if ((pData[i] == PPG_HEADER_ST1) && (pData[i+1] == PPG_HEADER_ST2)) {
            count++;
        }    
    }

    return count;
}

int dataFindGsensorHeaderIndex(const size_t length, const uint8_t* pData, uint8_t* pIdx) {
    if (length == 0) {
        return 0;
    }

    int cnt = 0;
    for (int i=0; i<length-1; i++) {
        if ((pData[i] == GSENSOR_HEADER_ST1) && (pData[i+1] == GSENSOR_HEADER_ST2)) {
            pIdx[cnt] = i;
            cnt++;
        }    
    }

    return cnt;
}

int dataFindPpgHeaderIndex(const size_t length, const uint8_t* pData, uint8_t* pIdx) {
    if (length == 0) {
        return -1;
    }    

    int cnt = 0;
    for (int i=0; i<length-1; i++) {
        if ((pData[i] == PPG_HEADER_ST1) && (pData[i+1] == PPG_HEADER_ST2)) {
            pIdx[cnt] = i;
            cnt++;
        }    
    }

    return cnt;
}

int dataFindHeaderIndex(const size_t length, const uint8_t* pData) {
    if (length == 0) {
        return -1;
    }

    int idx = -1;
    for (int i=0; i<length-1; i++) {
        if (((pData[i] == GSENSOR_HEADER_ST1) && (pData[i+1] == GSENSOR_HEADER_ST2)) ||
            ((pData[i] == PPG_HEADER_ST1)     && (pData[i+1] == PPG_HEADER_ST2))) {
            idx = i;
            break;
        }    
    }

    return idx;
}

int dataFindMaxEraseByteCount(const size_t length, const uint8_t* pData) {
    int maxCnt = 0;
    int cnt = 0;
    for (int i=0; i<length; i++) {
        if (pData[i] == 0xFF) {
            cnt++;
        } else {
            if (cnt > maxCnt) {
                maxCnt = cnt;
            }
            cnt = 0;
        }
    }

    return maxCnt;
}   

void data2Length(const uint8_t* pData, uint32_t* pLen) {
    uint32_t data[LENGTH_BUF_LEN];
    for (int i=0; i<LENGTH_BUF_LEN; i++) {
        data[i] = pData[i];
    }

    *pLen = data[LENGTH_IDX_DATA_START]   << 24 |
            data[LENGTH_IDX_DATA_START+1] << 16 |
            data[LENGTH_IDX_DATA_START+2] <<  8 |
            data[LENGTH_IDX_DATA_START+3]; 
}

void length2Data(const uint32_t length, uint8_t* pData) {
    pData[0]                       = LENGTH_HEADER_ST1;
    pData[1]                       = LENGTH_HEADER_ST2;
    pData[LENGTH_IDX_DATA_START]   = (length & 0xFF000000) >> 24;
    pData[LENGTH_IDX_DATA_START+1] = (length & 0x00FF0000) >> 16;
    pData[LENGTH_IDX_DATA_START+2] = (length & 0x0000FF00) >>  8;
    pData[LENGTH_IDX_DATA_START+3] = (length & 0x000000FF);
}

uint32_t gsensorData2Uint32(const uint8_t* pData) {
    uint32_t data32[3];
    for (int i=0; i<3; i++) {
        data32[i] = *(pData + i);
    }    
    
    const uint32_t ui = data32[0] << 8 |
                        data32[1];

    return ui;
}

uint32_t ppgData2Uint32(const uint8_t* pData) {
    uint32_t data32[3];
    for (int i=0; i<3; i++) {
        data32[i] = *(pData + i);
    }    
    
    const uint32_t ui = data32[0] << 16 |
                        data32[1] << 8  |
                        data32[2];

    return ui;
} 

int getPacketsNum(const int len) {
    return len / PPG_ALL_PACKETS_LENGTH;
}

void dataPrintHex(const char* str, const uint16_t len, const uint8_t* pData) {
    printk("%s\n", str);
    
    for (int i=0; i<len; i++) {
        printk("pData[%d]=0x%x\n", i, pData[i]);
    }
    
}

void dataPrintChar(const char* str, const uint16_t len, const uint8_t* pData) {
    printk("%s\n", str);
    for (int i=0; i<len; i++) {
        printk("pData[%d]=%c\n", i, pData[i]);
    }    
}

void ppgDataPrint(const char* str,
                       const ppg_channel_t ch,
                       const int idx,
                       const uint16_t len,
                       const uint8_t* pData) {
    printk("%s\n", str);
    const int iteration = len / PPG_ALL_PACKETS_LENGTH;
    for (int i=0; i<iteration; i++) {
        const uint32_t u32 = ppgData2Uint32(&pData[2+ch+i*PPG_ALL_PACKETS_LENGTH]);
        printk("pData[%d]: %u", i, u32);
    }    
}


