#include <stdio.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/timeutil.h>
#include "datetime.h"

// datetime
#define DATETIME_IDX_YEAR        4
#define DATETIME_IDX_MONTH       8
#define DATETIME_IDX_DAY         10
#define DATETIME_IDX_HOUR        12
#define DATETIME_IDX_MINUTE      14
#define DATETIME_IDX_SEC         16

uint32_t _ascii2Uint(const uint32_t ascii) {
    printk("%c %u\n", ascii, ascii - '0');
    return ascii - '0';
}

uint32_t _uint2Ascii(const uint32_t ui) {
    return ui + '0';
}

uint32_t nthDigit(uint32_t x, int n) {
    while (n--) {
        x /= 10;
    }
    return (x % 10);
}

datetime_t datetimeAddMs(const datetime_t* pDateTime, const int32_t ms) {
    datetime_t dateTime;
    const time_t timet = datetime2timet(pDateTime) + ms / 1000;
    timet2Datetime(&timet, &dateTime);
    
    return dateTime;
}


void data2Datetime(const uint8_t* pData, datetime_t* pDateTime) {
    uint32_t data[DATE_TIME_BUF_LEN];
    for (int i=0; i<DATE_TIME_BUF_LEN; i++) {
        data[i] = pData[i];
    }
    pDateTime->year   = _ascii2Uint(data[DATETIME_IDX_YEAR])   * 1000 + 
                        _ascii2Uint(data[DATETIME_IDX_YEAR+1]) *  100 +
                        _ascii2Uint(data[DATETIME_IDX_YEAR+2]) *   10 + 
                        _ascii2Uint(data[DATETIME_IDX_YEAR+3]);
    pDateTime->month  = _ascii2Uint(data[DATETIME_IDX_MONTH])  *   10 + 
                        _ascii2Uint(data[DATETIME_IDX_MONTH+1]);
    pDateTime->day    = _ascii2Uint(data[DATETIME_IDX_DAY])    *   10 + 
                        _ascii2Uint(data[DATETIME_IDX_DAY+1]);
    pDateTime->hour   = _ascii2Uint(data[DATETIME_IDX_HOUR])   *   10 + 
                        _ascii2Uint(data[DATETIME_IDX_HOUR+1]);
    pDateTime->minute = _ascii2Uint(data[DATETIME_IDX_MINUTE]) *   10 + 
                        _ascii2Uint(data[DATETIME_IDX_MINUTE+1]);
    pDateTime->sec    = _ascii2Uint(data[DATETIME_IDX_SEC])    *   10 + 
                        _ascii2Uint(data[DATETIME_IDX_SEC+1]);
    
} 

void datetime2Data(const datetime_t* pDateTime, uint8_t* pData) {
    datetimePrint("pDateTime", pDateTime);
    pData[0] = 0x54; // 'T'
    pData[1] = 0x49; // 'I'
    pData[2] = 0x4D; // 'M'
    pData[3] = 0x45; // 'E'
    pData[DATETIME_IDX_YEAR]     = _uint2Ascii(nthDigit(pDateTime->year, 3));
    pData[DATETIME_IDX_YEAR+1]   = _uint2Ascii(nthDigit(pDateTime->year, 2));
    pData[DATETIME_IDX_YEAR+2]   = _uint2Ascii(nthDigit(pDateTime->year, 1));
    pData[DATETIME_IDX_YEAR+3]   = _uint2Ascii(nthDigit(pDateTime->year, 0));
    printk("%u %u %u %u\n", nthDigit(pDateTime->year, 4), nthDigit(pDateTime->year, 3), nthDigit(pDateTime->year, 2), nthDigit(pDateTime->year, 1));  
    printk("%c %c %c %c\n", pData[DATETIME_IDX_YEAR], pData[DATETIME_IDX_YEAR+1], pData[DATETIME_IDX_YEAR+2], pData[DATETIME_IDX_YEAR+3]);  
    pData[DATETIME_IDX_MONTH]    = _uint2Ascii(nthDigit(pDateTime->month, 1));
    pData[DATETIME_IDX_MONTH+1]  = _uint2Ascii(nthDigit(pDateTime->month, 0));
    pData[DATETIME_IDX_DAY]      = _uint2Ascii(nthDigit(pDateTime->day, 1));
    pData[DATETIME_IDX_DAY+1]    = _uint2Ascii(nthDigit(pDateTime->day, 0));
    pData[DATETIME_IDX_HOUR]     = _uint2Ascii(nthDigit(pDateTime->hour, 1));
    pData[DATETIME_IDX_HOUR+1]   = _uint2Ascii(nthDigit(pDateTime->hour, 0));
    pData[DATETIME_IDX_MINUTE]   = _uint2Ascii(nthDigit(pDateTime->minute, 1));
    pData[DATETIME_IDX_MINUTE+1] = _uint2Ascii(nthDigit(pDateTime->minute, 0));
    pData[DATETIME_IDX_SEC]      = _uint2Ascii(nthDigit(pDateTime->sec, 1));
    pData[DATETIME_IDX_SEC+1]    = _uint2Ascii(nthDigit(pDateTime->sec, 0));
}

time_t datetime2timet(const datetime_t* pDateTime) {
    tm_t tm;
    datetime2tm(pDateTime, &tm);
    const time_t timet = timeutil_timegm(&tm);
    return timet;
}

void timet2Datetime(const time_t* timet, datetime_t* pDateTime) {
    const tm_t* pTm = gmtime(timet);
    tm2Datetime(pTm, pDateTime);
}

void datetime2tm(const datetime_t* pDateTime, tm_t* pTm) {
    pTm->tm_year = pDateTime->year - 1900;
    pTm->tm_mon  = pDateTime->month - 1;
    pTm->tm_mday = pDateTime->day;
    pTm->tm_hour = pDateTime->hour;
    pTm->tm_min  = pDateTime->minute;
    pTm->tm_sec  = pDateTime->sec;
}

void tm2Datetime(const tm_t* pTm, datetime_t* pDateTime) {
    pDateTime->year   = pTm->tm_year + 1900;;
    pDateTime->month  = pTm->tm_mon + 1;
    pDateTime->day    = pTm->tm_mday;
    pDateTime->hour   = pTm->tm_hour;
    pDateTime->minute = pTm->tm_min;
    pDateTime->sec    = pTm->tm_sec;
}

void datetimePrint(const char* str, const datetime_t* pTime) {
    printk("%s", str);
    printk("%d-%d-%d %d:%d:%d\n", 
                                         pTime->year,
                                         pTime->month,
                                         pTime->day,
                                         pTime->hour,
                                         pTime->minute,
                                         pTime->sec); 
} 


