/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#include <stdio.h>
#include <string.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/flash.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/sys/printk.h>


#if defined(CONFIG_BOARD_ADAFRUIT_FEATHER_STM32F405)
#define SPI_FLASH_TEST_REGION_OFFSET 0xf000
#elif defined(CONFIG_BOARD_ARTY_A7_ARM_DESIGNSTART_M1) || \
    defined(CONFIG_BOARD_ARTY_A7_ARM_DESIGNSTART_M3)
/* The FPGA bitstream is stored in the lower 536 sectors of the flash. */
#define SPI_FLASH_TEST_REGION_OFFSET \
    DT_REG_SIZE(DT_NODE_BY_FIXED_PARTITION_LABEL(fpga_bitstream))
#elif defined(CONFIG_BOARD_NPCX9M6F_EVB) || \
    defined(CONFIG_BOARD_NPCX7M6FB_EVB)
#define SPI_FLASH_TEST_REGION_OFFSET 0x7F000
#else
#define SPI_FLASH_TEST_REGION_OFFSET 0xff000
#endif
#define SPI_FLASH_SECTOR_SIZE        4096

int  flashNorInit(void)
{
    const struct device *flash_dev;
    int rc;

    flash_dev = DEVICE_DT_GET(DT_ALIAS(spi1flash0));

    if (!device_is_ready(flash_dev)) {
        printk("%s: device not ready.\n", flash_dev->name);
        return -1;
    }

    printk("\n%s SPI flash testing\n", flash_dev->name);
    printk("==========================\n");


    /* Write protection needs to be disabled before each write or
     * erase, since the flash component turns on write protection
     * automatically after completion of write and erase
     * operations.
     */
    printk  ("\nTest 1: Flash erase\n");

    /* Full flash erase if SPI_FLASH_TEST_REGION_OFFSET = 0 and
     * SPI_FLASH_SECTOR_SIZE = flash size
     */
    rc = flash_erase(flash_dev, SPI_FLASH_TEST_REGION_OFFSET,
             SPI_FLASH_SECTOR_SIZE);
    if (rc != 0) {
        printk("Flash erase failed! %d\n", rc);
    } else {
        printk("Flash erase succeeded!\n");
    }

    return 0;
}
