/*
 * Copyright (c) 2015 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <errno.h>
#include <string.h>
#include <zephyr/kernel.h>
#include <zephyr/sys/printk.h>
#include <zephyr/device.h>
#include <zephyr/drivers/i2c.h>
#include "error.h"
#include "i2c.h"

/**
 * @file Sample app using the Fujitsu MB85RC256V FRAM through I2C.
 */


static bool _gI2CInitialzed = false;
static uint8_t _gSlaveAddr = 0x5B;
const struct device *const _gI2cDev = DEVICE_DT_GET(DT_NODELABEL(i2c0));
#if 0
static int _i2cWriteBytes(const uint8_t addr, uint8_t *data, const uint32_t num_bytes) {
    uint8_t wr_addr[1];
    struct i2c_msg msgs[2];

    /* FRAM address */
    //wr_addr[0] = (addr >> 8) & 0xFF;
    //wr_addr[1] = addr & 0xFF;
    wr_addr[0] = addr & 0xFF;

    /* Setup I2C messages */

    /* Send the address to write to */
    msgs[0].buf = wr_addr;
    msgs[0].len = 2U;
    msgs[0].flags = I2C_MSG_WRITE;

    /* Data to be written, and STOP after this. */
    msgs[1].buf = data;
    msgs[1].len = num_bytes;
    msgs[1].flags = I2C_MSG_WRITE | I2C_MSG_STOP;

    return i2c_transfer(_gI2cDev, &msgs[0], 2, _gSlaveAddr);
}

static int _i2cReadBytes(const uint8_t addr, uint8_t *data, uint32_t num_bytes) {
    uint8_t wr_addr[1];
    struct i2c_msg msgs[2];

    /* Now try to read back from FRAM */

    /* FRAM address */
    //wr_addr[0] = (addr >> 8) & 0xFF;
    //wr_addr[1] = addr & 0xFF;
    wr_addr[0] = addr & 0xFF;

    /* Setup I2C messages */

    /* Send the address to read from */
    msgs[0].buf = wr_addr;
    msgs[0].len = 2U;
    msgs[0].flags = I2C_MSG_WRITE;

    /* Read from device. STOP after this. */
    msgs[1].buf = data;
    msgs[1].len = num_bytes;
    msgs[1].flags = I2C_MSG_READ | I2C_MSG_STOP;

    return i2c_transfer(_gI2cDev, &msgs[0], 2, _gSlaveAddr);
}
#endif

int i2cInit(void) {
    int ret;

    if (!device_is_ready(_gI2cDev)) {
        printk("I2C: Device is not ready.\n");
        return ERROR_I2C_INITIALIZE_FAIL;
    }
    printk("I2C: Device is ready.\n");

   // const uint32_t i2cCfg = I2C_MODE_CONTROLLER | I2C_SPEED_SET(I2C_SPEED_FAST);
   // ret = i2c_configure(_gI2cDev, i2cCfg);
   // if (ret < 0) {
   //     printk("I2C: Configure failed err=%d\n", ret);
   // }
   // printk("I2C: Configure okay.\n");

    #define AFE4900_I2C_ADDRESS  0x5B
    i2cSlaveAddr(AFE4900_I2C_ADDRESS);
    uint8_t buf[3];
    ret = i2c_burst_read(_gI2cDev, AFE4900_I2C_ADDRESS, 0x28, buf, 3);
    if (ret < 0) {
        printk("I2C: i2c_burst_read() failed err=%d\n", ret);
    } else {
        printk("0x%x 0x%x 0x%x\n", buf[2], buf[1], buf[0]);
    }

    return 0;
}


void i2cUninit(void) {
    if (_gI2CInitialzed) {
        
        //nrf_drv_twi_uninit(&m_twi_master);
        _gI2CInitialzed = false;
    }
}

void i2cSlaveAddr(const uint8_t addr) {
    _gSlaveAddr = addr;
}

int i2cWriteReg(const uint8_t reg, uint8_t data) {
    const int ret = i2c_burst_write(_gI2cDev, _gSlaveAddr, reg, &data, 1);
    if (ret < 0) {
        printk("I2C: i2c_burst_write() failed err=%d\n", ret);
        return ERROR_I2C_WRITE_FAIL;
    }

    return ret;
}

int i2cWriteData(const uint8_t reg, uint8_t *data, const uint32_t len) {
    const int ret = i2c_burst_write(_gI2cDev, _gSlaveAddr, reg, data, len);
    if (ret < 0) {
        printk("I2C: i2c_burst_write() failed err=%d\n", ret);
        return ERROR_I2C_WRITE_FAIL;
    }

    return ret;
}
#if 0
int i2cReadBatchData(const uint8_t reg, uint8_t *data, const uint32_t len) {
    const int ret = i2c_burst_read(_gI2cDev, _gSlaveAddr, reg, data, len);
    if (ret < 0) {
        printk("I2C: i2c_burst_read() failed err=%d\n", ret);
        return ERROR_I2C_READ_FAIL;
    }

    return ret;
}
#endif

int i2cReadData(const uint8_t reg, uint8_t* data, const uint32_t len) {
    const int ret = i2c_burst_read(_gI2cDev, _gSlaveAddr, reg, data, len);
    if (ret < 0) {
        printk("I2C: i2c_burst_read() failed err=%d\n", ret);
        return ERROR_I2C_READ_FAIL;
    }

    return ret;
}


uint8_t i2cReadReg(uint8_t reg) {
    uint8_t data;

    const int ret = i2c_burst_read(_gI2cDev, _gSlaveAddr, reg, &data, 1);
    if (ret < 0) {
        printk("I2C: i2c_burst_read() failed err=%d\n", ret);
        return ERROR_I2C_READ_FAIL;
    } 

    return data;
}
