#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/led.h>
#include <zephyr/sys/printk.h>
#include "error.h"

static const struct gpio_dt_spec _gLedR = GPIO_DT_SPEC_GET(DT_ALIAS(ledr), gpios);
static const struct gpio_dt_spec _gLedG = GPIO_DT_SPEC_GET(DT_ALIAS(ledg), gpios);
static const struct gpio_dt_spec _gLedB = GPIO_DT_SPEC_GET(DT_ALIAS(ledb), gpios);

int ledInit(void) {
    int ret;
    // R
    if (!device_is_ready(_gLedR.port)) {
        printk("LED device %s is not ready\n", _gLedR.port->name);
        return ERROR_LED_INITIALIZE_FAIL;
    }

    ret = gpio_pin_configure_dt(&_gLedR, GPIO_OUTPUT);
    if (ret < 0) {
        printk("Failed to configure the LED pin, error: %d\n", ret);
        return ERROR_LED_INITIALIZE_FAIL;
    }

    // G
    if (!device_is_ready(_gLedG.port)) {
        printk("LED device %s is not ready\n", _gLedG.port->name);
        return ERROR_LED_INITIALIZE_FAIL;
    }

    ret = gpio_pin_configure_dt(&_gLedG, GPIO_OUTPUT);
    if (ret < 0) {
        printk("Failed to configure the LED pin, error: %d\n", ret);
        return ERROR_LED_INITIALIZE_FAIL;
    }

    // B
    if (!device_is_ready(_gLedB.port)) {
        printk("LED device %s is not ready\n", _gLedB.port->name);
        return ERROR_LED_INITIALIZE_FAIL;
    }

    ret = gpio_pin_configure_dt(&_gLedB, GPIO_OUTPUT);
    if (ret < 0) {
        printk("Failed to configure the LED pin, error: %d\n", ret);
        return ERROR_LED_INITIALIZE_FAIL;
    }

    return 0;
}

int ledClearR(void) {
    gpio_pin_set_dt(&_gLedR, 0);

    return 0;
}

int ledClearG(void) {
    gpio_pin_set_dt(&_gLedG, 0);

    return 0;
}

int ledClearB(void) {
    gpio_pin_set_dt(&_gLedB, 0);

    return 0;
}

int ledClearRGB(void) {
    ledClearR();
    ledClearG();
    ledClearB();

    return 0;
}

int ledSetR(void) {
    gpio_pin_set_dt(&_gLedR, 1);

    return 0;
}

int ledSetG(void) {
    gpio_pin_set_dt(&_gLedG, 1);

    return 0;
}

int ledSetB(void) {
    gpio_pin_set_dt(&_gLedB, 1);

    return 0;
}

int ledSetRGB(void) {
    ledSetR();
    ledSetG();
    ledSetB();

    return 0;
}

