/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include <hal/nrf_rtc.h>
#include <zephyr/kernel.h>
#include "localtime.h"
#include "rtc.h"

datetime_t _gDateTime;
uint32_t   _gRtcTick;

uint32_t ltDiffTick(const uint32_t t1, const uint32_t t2) {
    const uint32_t tmax = RTC_COUNTER_MAX;

    if (t1 > t2) {
        return t2 + (tmax - t1) + 1;
    }

    return t2 - t1;
}

/* capture initial time stamp */
void ltInit(void) {
}

datetime_t ltGetDatetime(void) {
    //const int64_t spentMs  = k_uptime_delta(&_gTimeStamp);
    const uint32_t now       = nrf_rtc_counter_get(NRF_RTC0);
    const uint32_t diffTicks = ltDiffTick(_gRtcTick, now);
    const uint32_t spentMs   = RTC_TICKS_TO_MS(diffTicks);
    datetime_t datetime      = datetimeAddMs(&_gDateTime, spentMs);

    return datetime;
}

void ltSetDatetime(const datetime_t* dateTime) {
    _gDateTime = *dateTime;
    //_gTimeStamp = k_uptime_get();
    _gRtcTick = nrf_rtc_counter_get(NRF_RTC0);
}

void ltPrintCurrentDatetime(void) {
    const datetime_t datetime = ltGetDatetime();
    datetimePrint("now", &datetime);
}
